package com.yoho.opsmanager.common.util;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yoho.opsmanager.common.model.ClientReport.ClientPatchModel;
import com.yoho.opsmanager.common.model.ClientReport.ClientReportErrorModel;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Base64;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Created by zhengyouwei on 2016/6/21.
 */
public class ClientPatchUtil {

    private static ObjectMapper mapper = new ObjectMapper();

    private static Pattern p = Pattern.compile("method=[a-zA-Z[\\\\.]]+"); // 匹配方法

    public static String getTableName() {
        return "client_patch";
    }

    private static Logger logger = LoggerFactory.getLogger(ClientPatchUtil.class);

    public static List<ClientPatchModel> getPatchsFromReport(String mlogs, String client) {
        List<ClientPatchModel> list = new ArrayList<>();

        String osv = "0";
        String dm = "other";
        String av = "other";
        //运营商
        try {
            JSONObject json = JSONObject.fromObject(mlogs);

            if (json.has("status") && CollectionUtils.isNotEmpty(json.getJSONArray("status"))) {
                JSONArray status = json.getJSONArray("status");
                Map<String, Object> map = readValue(status.get(0).toString());
                if (map.containsKey("av")) {
                    av = String.valueOf(map.get("av"));
                }
            }

            if (json.has("device")) {
                JSONObject device = json.getJSONObject("device");
                Map<String, Object> map = readValue(device.toString());
                if (map.containsKey("osv")) {
                    osv = String.valueOf(map.get("osv"));
                }
                if (map.containsKey("dm")) {
                    dm = String.valueOf(map.get("dm"));
                }
                if (client == null) {
                    client = String.valueOf(map.get("os"));
                }
            }

            if (json.has("patch") && CollectionUtils.isNotEmpty(json.getJSONArray("patch"))) {
                JSONArray array = json.getJSONArray("patch");
                for (Object error : array) {
                    Map<String, Object> map = readValue(error.toString());
                    ClientPatchModel clientPatchModel = new ClientPatchModel(osv, dm, av);
                    clientPatchModel.setClient(client);

                    if (map.containsKey("uid") && map.get("uid") != null) {
                        clientPatchModel.setUid(String.valueOf(map.get("uid")));
                    }
                    long value = Long.valueOf(String.valueOf(map.get("ts")));
                    clientPatchModel.setTs(DateFormatUtil.parseLongTo14(value));
                    Map<String, Object> paramMap = (Map<String, Object>) map.get("param");
                    for (String key : paramMap.keySet()) {
                        switch (key) {
                            case "patcht":
                                String patcht = String.valueOf(paramMap.get(key));
                                clientPatchModel.setPatcht(patcht);
                                break;
                            case "patchv":
                                String patchv = String.valueOf(paramMap.get(key));
                                clientPatchModel.setPatchv(patchv);
                                break;
                        }
                    }
                    list.add(clientPatchModel);
                }
            }
        } catch (Exception e) {
            logger.error("getPatchFromReport faile {} ", e.toString());
        }
        return list;
    }

    /**
     * json转化为Map
     *
     * @param json
     * @return 2016年5月19日下午4:21:06
     */
    public static Map<String, Object> readValue(String json) {
        try {
            return mapper.readValue(json.toString(), new TypeReference<Map<String, Object>>() {
            });
        } catch (Exception e) {
            return null;
        }
    }

}
