package com.yoho.opsmanager.common.util;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yoho.opsmanager.common.model.ClientReport.ClientReportErrorModel;
import net.sf.json.JSON;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Base64;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.zip.GZIPOutputStream;

/**
 * Created by zhengyouwei on 2016/6/21.
 */
public class ClientReportUtil {

    private static ObjectMapper mapper = new ObjectMapper();

    private static Pattern p = Pattern.compile("method=[a-zA-Z[\\\\.]]+"); // 匹配方法

    public static String getTableName(String et) {
        return "client_" + et;
    }

    public static String getHourName(String et, String client) {
        return "hour_" + client + "_" + et;
    }

    public static String getDayName(String et, String client) {
        return "day_" + client + "_" + et;
    }

    private static Logger logger = LoggerFactory.getLogger(ClientReportUtil.class);

        private static String ETS = "PoC,NETWORK,IMAGELOAD,CRASH,RNException,WVNOTHTTPS,WVMETA,CDNCHANGE,CENTERCHANGE,ULOGIN,LPOC";

    public static List<ClientReportErrorModel> getErrorsFromReport(String mlogs, String client) {
        List<ClientReportErrorModel> list = new ArrayList<>();

        String osv = "0";
        String dm = "other";
        String av = "other";
        //app版本号
        String net = "0";
        //网络
        String ca = "0";
        //运营商
        String udid = "0";
        //运营商
        try {
            JSONObject json = JSONObject.fromObject(mlogs);

            if (json.has("status") && CollectionUtils.isNotEmpty(json.getJSONArray("status"))) {
                JSONArray status = json.getJSONArray("status");
                Map<String, Object> map = readValue(status.get(0).toString());
                if (map.containsKey("av")) {
                    av = String.valueOf(map.get("av"));
                }
                if (map.containsKey("net")) {
                    net = String.valueOf(map.get("net"));
                }
                if (map.containsKey("ca")) {
                    ca = String.valueOf(map.get("ca"));
                }
            }

            if (json.has("device")) {
                JSONObject device = json.getJSONObject("device");
                Map<String, Object> map = readValue(device.toString());
                if (map.containsKey("osv")) {
                    osv = String.valueOf(map.get("osv"));
                }
                if (map.containsKey("dm")) {
                    dm = String.valueOf(map.get("dm"));
                }
                if (client == null) {
                    client = String.valueOf(map.get("os"));
                }
                if (map.containsKey("udid")) {
                    udid = String.valueOf(map.get("udid"));
                }
            }

            if (json.has("errors") && CollectionUtils.isNotEmpty(json.getJSONArray("errors"))) {
                JSONArray array = json.getJSONArray("errors");
                for (Object error : array) {
                    Map<String, Object> map = readValue(error.toString());
                    ClientReportErrorModel clientReportErrorModel = new ClientReportErrorModel(osv, dm, av, net, ca);
                    clientReportErrorModel.setDevClass(client);
                    clientReportErrorModel.setClient(client);
                    clientReportErrorModel.setUdid(udid);
                    String et = String.valueOf(map.get("et"));
                    if (!ETS.contains(et)) {
                        continue;
                    }

                    clientReportErrorModel.setEt(et);//PoC NETWORK IMAGELOAD

                    if (map.containsKey("uid") && map.get("uid") != null) {
                        clientReportErrorModel.setUid(String.valueOf(map.get("uid")));
                    }
                    clientReportErrorModel.setTec(String.valueOf(map.get("tec")));
                    long value = Long.valueOf(String.valueOf(map.get("ts")));
                    clientReportErrorModel.setTs(DateFormatUtil.parseLongTo14(value));
                    Map<String, Object> paramMap = (Map<String, Object>) map.get("param");
                    if ("IMAGELOAD".equals(et)) {
                        for (String key : paramMap.keySet()) {
                            switch (key) {
                                case "method":
                                    String method = String.valueOf(paramMap.get(key));
                                    clientReportErrorModel.setMethod(method);
                                    break;
                                case "body":
                                    String bodyvalue = String.valueOf(paramMap.get(key));
                                    clientReportErrorModel.setBody(bodyvalue);
                                    break;
                                case "ei":
                                    String eivalue = String.valueOf(paramMap.get(key));
                                    clientReportErrorModel.setEi(eivalue);
                                    break;
                                case "ec":
                                    String ecvalue = String.valueOf(paramMap.get(key));
                                    if ("0".equals(ecvalue) && "ios".equals(client)) {
                                        ecvalue = "-1";
                                    }
                                    clientReportErrorModel.setEc(ecvalue);
                                    break;
                                case "response":
                                    String response = String.valueOf(paramMap.get(key));
                                    clientReportErrorModel.setResponse(response);
                                    clientReportErrorModel.setExceptionType(base64Encode(getExceptionType(response)));
                                    break;
                            }

                        }
                    } else if ("CRASH".equals(et)) {
                        for (String key : paramMap.keySet()) {
                            switch (key) {
                                case "st":
                                    String stvalue = String.valueOf(paramMap.get(key));
                                    clientReportErrorModel.setSt(stvalue);
                                    if ("android".equals(clientReportErrorModel.getClient().toLowerCase())) {
                                        clientReportErrorModel.setExceptionType(base64Encode(getExceptionType(stvalue)));
                                    } else {
                                        clientReportErrorModel.setExceptionType(base64Encode("CRASH"));
                                    }
                                    break;
                            }
                        }
                    } else if ("PoC".equals(et) || "NETWORK".equals(et)) {
                        String methodevent = null;
                        clientReportErrorModel.setEi(String.valueOf(paramMap.get("ei")));
                        for (String key : paramMap.keySet()) {
                            switch (key) {
                                case "method":
                                    String method = String.valueOf(paramMap.get(key));
                                    clientReportErrorModel.setMethod(method);
                                    break;
                                case "body":
                                    String bodyvalue = String.valueOf(paramMap.get(key));
                                    if (methodevent == null) {
                                        methodevent = methodMatch(bodyvalue);
                                    }
                                    clientReportErrorModel.setBody(bodyvalue);
                                    break;
                                case "ei":
                                    String eivalue = String.valueOf(paramMap.get(key));
                                    if (methodevent == null) {
                                        methodevent = methodMatch(eivalue);
                                    }
                                    clientReportErrorModel.setEi(eivalue);
                                    break;
                                case "ec":
                                    String ecvalue = String.valueOf(paramMap.get(key));
                                    if ("0".equals(ecvalue) && "ios".equals(client)) {
                                        ecvalue = "-1";
                                    }
                                    clientReportErrorModel.setEc(ecvalue);
                                    if ("PoC".equals(et)) {
                                        clientReportErrorModel.setExceptionType(base64Encode(ecvalue));
                                    }
                                    break;
                                case "response":
                                    String response = String.valueOf(paramMap.get(key));
                                    clientReportErrorModel.setResponse(response);
                                    if ("NETWORK".equals(et)) {
                                        if ("android".equals(clientReportErrorModel.getClient().toLowerCase()) && clientReportErrorModel.getResponse().startsWith("failed")) {
                                            clientReportErrorModel.setExceptionType(base64Encode("connect timeout"));
                                        } else {
                                            clientReportErrorModel.setExceptionType(base64Encode(getExceptionType(response)));
                                        }
                                    }
                                    break;
                            }

                        }
                        if (methodevent == null) {//methodevent 还是为NULL 取ei？前
                            String eivalue = String.valueOf(paramMap.get("ei"));
                            if (eivalue.indexOf("?") != -1) {
                                methodevent = eivalue.substring(0, eivalue.indexOf("?"));
                            } else {
                                methodevent = eivalue;
                            }
                        }
                        clientReportErrorModel.setMethodevent(methodevent);

                    } else {
                        clientReportErrorModel.setEi(String.valueOf(paramMap.get("ei")));
                        clientReportErrorModel.setParam(JSONObject.fromObject(paramMap).toString());
                        clientReportErrorModel.setExceptionType(base64Encode(et));
                    }
                    list.add(clientReportErrorModel);
                }
            }
        } catch (Exception e) {
            logger.error("getErrorsFromReport faile {} ", e.toString());
        }

        return list;
    }

    /**
     * json转化为Map
     *
     * @param json
     * @return 2016年5月19日下午4:21:06
     */
    public static Map<String, Object> readValue(String json) {
        try {
            return mapper.readValue(json.toString(), new TypeReference<Map<String, Object>>() {
            });
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * 匹配method
     *
     * @param bodyvalue
     * @return
     */

    private static String methodMatch(String bodyvalue) {
        Matcher matcher = p.matcher(bodyvalue); // 操作的字符串
        boolean find = matcher.find();
        if (find) {
            return matcher.group();
        }
        return null;
    }

    public static String base64Encode(String str) {
        if (StringUtils.isBlank(str)) {
            return null;
        }
        return Base64.getEncoder().encodeToString(str.getBytes(StandardCharsets.UTF_8));

    }

    public static String base64Decode(String str) {
        if (StringUtils.isBlank(str)) {
            return null;
        }
        return new String(Base64.getDecoder().decode(str), StandardCharsets.UTF_8);

    }

    public static String getExceptionType(String source) {
        String exception;
        if (source.length() < 50) {
            exception = source;
        } else if (source.indexOf(":") != -1) {
            exception = source.substring(0, source.indexOf(":"));
        } else if (source.indexOf("\\(") != -1) {
            exception = source.substring(0, source.indexOf("\\("));
        } else {
            exception = source.substring(0, 50);
        }
        if (exception.length() > 50) {
            return source.substring(0, 50);
        }
        return exception;
    }

    public static void main(String[] args) throws IOException {
        String a = ClientReportUtil.base64Encode("请求超时。");
        System.out.println(a);
        String b = ClientReportUtil.base64Decode(a);
        System.out.println(b);
    }

}
