package com.yoho.opsmanager.dao.mapper.impl;

import com.yoho.opsmanager.common.contants.DataBaseContants;
import com.yoho.opsmanager.dao.InluxDBSingle;
import com.yoho.opsmanager.dao.mapper.MonitorAlarmMapper;
import org.influxdb.dto.BatchPoints;
import org.influxdb.dto.Point;
import org.influxdb.dto.QueryResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service("monitorAlarmMapper")
public class MonitorAlarmMapperImpl extends InfluxDBQuery implements MonitorAlarmMapper {

	@Autowired
	private InluxDBSingle inluxDBSingle;

	/**
	 * 插入告警短信
	 * @param influxDBName
	 * @param alarmType
	 * @param snsContent
	 * @param alarmInfo
	 */
	@Override
	public void insertAlarmMsg(String influxDBName, String alarmType, String snsContent, String alarmInfo, String sendStatus,String result) {
		BatchPoints batchPoints = BatchPoints
				.database(DataBaseContants.YOMO_MONITOR).tag("event", "alarm").retentionPolicy("default")
				.build();
		Point point = Point.measurement("monitor_alarm")
				.tag("type", alarmType)
				.tag("send_status", sendStatus)
				.addField("sms_content", snsContent)
				.addField("result", result)
				.addField("alarm_info", alarmInfo).build();
		batchPoints.point(point);
		if(influxDBName == null){
			influxDBName = inluxDBSingle.getDBNames().get(0);
		}
		inluxDBSingle.getInfluxDBByName(influxDBName).getInfluxDB()
				.write(batchPoints);
	}

	/**
	 * 查询type的tag-value
	 * @param influxDBName
	 * @return
	 */
	@Override
	public QueryResult selectTagValuesType(String influxDBName) {
		String command = "SHOW TAG VALUES FROM monitor_alarm WITH KEY = type";

		return query(influxDBName, command, DataBaseContants.YOMO_MONITOR,
				"MonitorAlarmMapperImpl", "selectTagValuesType");
	}

	/**
	 * 查询指定influxdb type的告警短信内容
	 * @param influxDBName
	 * @param type
	 * @return
	 */
	@Override
	public QueryResult selectAlarmMsg(String influxDBName, String type, int minute) {
		String command = "SELECT sms_content, info, send_status FROM monitor_alarm WHERE event = 'alarm'" +
						" AND time > now() - " + minute + "m" +
						" AND type = '" + type + "' ORDER BY time DESC";
		return query(influxDBName, command, DataBaseContants.YOMO_MONITOR,
				"MonitorAlarmMapperImpl", "selectAlarmMsg");
	}

	/**
	 * 查询指定influxdb type的告警短信内容，并且指定offset
	 *
	 * @param influxDBName
	 * @param type
	 * @param offsetCount
	 * @return
	 */
	@Override
	public QueryResult selectAlarmMsgByPagination(String influxDBName, String type, int limitCount, int offsetCount) {
		String command = "SELECT sms_content, info, send_status FROM monitor_alarm WHERE event = 'alarm'" +
						" AND type = '" + type + "' ORDER BY time DESC LIMIT " + limitCount +
						" OFFSET " + offsetCount;
		return query(influxDBName, command, DataBaseContants.YOMO_MONITOR,
				"MonitorAlarmMapperImpl", "selectAlarmMsgByPagination");
	}

	@Override
	public QueryResult selectAlarmMsgByPagination(String influxDBName, int limitCount, int offsetCount) {
		String command = "SELECT sms_content, info, send_status FROM monitor_alarm WHERE event = 'alarm'" +
				" ORDER BY time DESC LIMIT " + limitCount +
				" OFFSET " + offsetCount;
		return query(influxDBName, command, DataBaseContants.YOMO_MONITOR,
				"MonitorAlarmMapperImpl", "selectAlarmMsgByPagination");
	}


	/**
	 * 查询指定数据库指定type的数据count
	 *
	 * @param influxDBName
	 * @param type
	 * @return
	 */
	@Override
	public QueryResult selectAlarmMsgCount(String influxDBName, String type) {
		String command = "SELECT COUNT(sms_content) FROM monitor_alarm " +
				"WHERE event = 'alarm' AND type = '" + type + "'";
		return query(influxDBName, command, DataBaseContants.YOMO_MONITOR,
				"MonitorAlarmMapperImpl", "selectAlarmMsgCount");
	}
	/**
	 * 查询指定数据库指定type的数据count
	 *
	 * @param influxDBName
	 * @return
	 */
	@Override
	public QueryResult selectAlarmMsgCount(String influxDBName) {
		String command = "SELECT COUNT(sms_content) FROM monitor_alarm " +
				"WHERE event = 'alarm'";
		return query(influxDBName, command, DataBaseContants.YOMO_MONITOR,
				"MonitorAlarmMapperImpl", "selectAlarmMsgCount");
	}
}
