package com.yoho.opsmanager.global.service.impl;

import com.alibaba.fastjson.JSON;
import com.yoho.opsmanager.common.config.SendsmsConfig;
import com.yoho.opsmanager.common.contants.InfluxDBName;
import com.yoho.opsmanager.common.model.QcloudSms;
import com.yoho.opsmanager.common.util.MD5Util;
import com.yoho.opsmanager.dao.mapper.MonitorAlarmMapper;
import com.yoho.opsmanager.global.service.AlarmMsgService;
import com.yoho.opsmanager.global.service.HttpRestClientService;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by yoho on 2016/5/3.
 */
@Service("alarmMsgService")
public class AlarmMsgServiceImpl implements AlarmMsgService {

    @Autowired
    private HttpRestClientService httpRestClientService;

    @Autowired
    private SendsmsConfig sendsmsConfig;

    @Autowired
    private MonitorAlarmMapper monitorAlarmMapper;

    private Logger logger = LoggerFactory.getLogger(AlarmMsgServiceImpl.class);


    /**
     * 发送短信
     *
     * @throws UnsupportedEncodingException
     */
    @Override
    public boolean sendSms(String type, String content, String mobile) {
        logger.info("Send msg type {} content {} moniole {}...", type, content, mobile);
        if (StringUtils.isBlank(content)) {
            return false;
        }
        String result = sendSingleSms(content, mobile);
        boolean sendStatus = true;
        if (result == null) {
            sendStatus = false;
        }

        try {
            monitorAlarmMapper.insertAlarmMsg(InfluxDBName.AWS, type, content, "  ", sendStatus ? "successed" : "faild", result);
        } catch (Exception e) {
            logger.error("insert sms into influxdb failed", e);
        }

        return sendStatus;
    }

    /**
     * 发送短信
     *
     * @return
     */
    private String sendSingleSms(String content, String mobile) {

        try {
            QcloudSms qcloudSms = new QcloudSms();
            List<QcloudSms.Tel> telList = new ArrayList<>();
            String[] mobiles = mobile.split(",");
            QcloudSms.Tel tel;
            for (String mob : mobiles) {
                tel = new QcloudSms.Tel();
                tel.setPhone(mob);
                telList.add(tel);
            }
            qcloudSms.setTel(telList);
            qcloudSms.setMsg("故障：" + content);
            qcloudSms.setSig(MD5Util.encryption(sendsmsConfig.getQcloudSmsKey() + mobile));
            String result = httpRestClientService.doPostStringJson(sendsmsConfig.getQcloudSmsUrl(), JSON.toJSONString(qcloudSms));
            return result;
        } catch (Exception e) {
            logger.error("send sms failed", e);
            return null;
        }
    }
}
