package com.yoho.opsmanager.global.service.impl;

import com.yoho.opsmanager.global.service.HttpRestClientService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.*;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.util.UriComponentsBuilder;

import java.net.URI;
import java.nio.charset.Charset;
import java.util.Map;

@Service("httpRestClientService")
public class HttpRestClientServiceImpl implements HttpRestClientService {

    private static final Logger logger = LoggerFactory
            .getLogger(HttpRestClientServiceImpl.class);

    @Autowired
    private RestTemplate restTemplate;

    private final static String ASK = "?";

    private final static String AND = "&";

    @Override
    public String doPostStringJson(String url, String args) {
        try {
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(new MediaType("application" ,"json", Charset.forName("UTF-8")));
            HttpEntity<String> entity = new HttpEntity<String>(args, headers);
            String result = restTemplate.postForObject(url, entity,
                    String.class);
            return result;
        } catch (Exception e) {
            logger.error("doPostStringJson failed!error:{},url:{}", e,url);
            return null;
        }
    }

    @Override
    public String doGet(final String uri, final Map<String, String> content) {
        String result = null;
        try {
            if (content != null) {
                StringBuilder url = new StringBuilder(uri);
                url.append(ASK);
                for (Map.Entry<String, String> entry : content.entrySet()) {
                    url.append(entry.getKey() + "=" + entry.getValue() + AND);
                }
                result = restTemplate.getForObject(url.toString().substring(0, url.length() - 1), String.class);
            } else {
                result = restTemplate.getForObject(uri, String.class);
            }
        } catch (Exception e) {
            logger.error("doGet failed!error:{},url:{}", e,uri);
        }
        return result;
    }

    @Override
    public <T> T postForObject(String url, Object request, Class<T> responseType) {
        try {
            return restTemplate.postForObject(url, request,
                    responseType);
        } catch (Exception e) {
            logger.error("postForObject failed!error:{},url:{}", e,url);
            return null;
        }
    }

    @Override
    public <T> T get(String path, Map request, Class<T> responseType) {

        logger.debug("begin to do http get. path:{}, request params:{}", path, request);

        //add accept json
        HttpHeaders headers = new HttpHeaders();
        headers.set("Accept", MediaType.APPLICATION_JSON_VALUE);
        HttpEntity<?> entity = new HttpEntity<>(headers);

        //url
        UriComponentsBuilder builder = UriComponentsBuilder.fromHttpUrl(path);

        //添加参数
        Map<String, ?> params = (Map<String, ?>) request;
        if (params != null && params.size() > 0) {
            for (Map.Entry<String, ?> urlParam : params.entrySet()) {
                builder.queryParam(urlParam.getKey(), urlParam.getValue());
            }
        }
        final URI getURI = builder.build().encode().toUri();

        try {
            HttpEntity<T> response = restTemplate.exchange(getURI, HttpMethod.GET, entity, responseType);
            logger.debug("end to do http get. path:{}, request params:{}. response body: {}", path, request, response.getBody());
            return response.getBody();
        } catch (Exception e) {
            logger.error("get failed!url:"+path, e);
            return null;
        }
    }

    @Override
    public String doGet(String uri, Map<String, String> content, Map<String, String> header) {
        String result = null;
        try {
            HttpHeaders headers = new HttpHeaders();
            for (Map.Entry<String, String> entry : header.entrySet()) {
                headers.set(entry.getKey(),entry.getValue());
            }
            HttpEntity<String> entity = new HttpEntity<String>(null, headers);

            if (content != null) {
                StringBuilder url = new StringBuilder(uri);
                url.append(ASK);
                for (Map.Entry<String, String> entry : content.entrySet()) {
                    url.append(entry.getKey() + "=" + entry.getValue() + AND);
                }
                ResponseEntity<String> response = restTemplate.exchange(url.toString().substring(0, url.length() - 1), HttpMethod.GET,entity,String.class);
                return response.getBody();
            } else {
                result = restTemplate.exchange(uri, HttpMethod.GET, entity, String.class).getBody();
            }
        } catch (Exception e) {
            logger.error("doGet failed!error:{},url:{}", e,uri);
        }
        return result;
    }

}
