package com.yoho.opsmanager.other.clientReport;

import com.yoho.opsmanager.common.model.ClientReport.ClientReportErrorModel;
import com.yoho.opsmanager.common.util.ApplicationUtil;
import com.yoho.opsmanager.common.util.ClientReportUtil;
import com.yoho.opsmanager.common.util.ReportCount;
import com.yoho.opsmanager.dao.mapper.ClientReportMapper;
import com.yoho.opsmanager.redis.service.ClientReportRedis;
import org.apache.commons.lang.StringUtils;
import org.influxdb.dto.Point;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.concurrent.TimeUnit;

/**
 * Created by zhengyouwei on 2016/8/24.
 */
public class ClientWriteRunnable implements Runnable {

    private String _mlogs;

    private String client;

    private static Random random = new Random();

    private Logger log = LoggerFactory.getLogger(getClass());

    public ClientWriteRunnable(String _mlogs, String client) {
        this._mlogs = _mlogs;
        this.client = client;
    }

    @Override
    public void run() {
        ReportCount.getAndIncrement();
        List<ClientReportErrorModel> list = ClientReportUtil.getErrorsFromReport(_mlogs, client);
        if (list.size() == 0) {
            return;
        }

        try {
            ClientReportRedis clientReportRedis = (ClientReportRedis) ApplicationUtil.getBean(ClientReportRedis.class);
            ClientReportMapper errorReportMapper = (ClientReportMapper) ApplicationUtil.getBean(ClientReportMapper.class);

            for (ClientReportErrorModel clientReportErrorModel : list) {

            	//client_error_report这个表，只包含IMAGELOAD、CRASH、PoC、NETWORK类型
                org.influxdb.dto.Point.Builder build = Point.measurement(ClientReportUtil.getTableName(clientReportErrorModel.getEt()));
                //client_error_data这个表包含所有类型
                org.influxdb.dto.Point.Builder buildData = Point.measurement(ClientReportUtil.getTableName(clientReportErrorModel.getEt()));

                Map<String, String> tag = new HashMap<>();
                tag.put("client", client);
                tag.put("osv", clientReportErrorModel.getOsv());
                tag.put("dm", clientReportErrorModel.getDm());
                tag.put("av", clientReportErrorModel.getAv());
                tag.put("ca", clientReportErrorModel.getCa());
                build.addField("net", clientReportErrorModel.getNet());
                build.addField("udid", clientReportErrorModel.getUdid());
                tag.put("et", clientReportErrorModel.getEt());
                tag.put("exceptionType", clientReportErrorModel.getExceptionType());
                //插入原始数据表
                for (Map.Entry<String, String> entry : tag.entrySet()) {
                    if (StringUtils.isBlank(entry.getValue())) {
                        tag.put(entry.getKey(),"null");
                    }
                }
                buildData.tag(tag);
                if ("IMAGELOAD".equals(clientReportErrorModel.getEt())) {
                    tag.put("ei", clientReportErrorModel.getEi());
                    //插入原始数据表
                    buildData.addField("ei", clientReportErrorModel.getEi());
                } else if ("CRASH".equals(clientReportErrorModel.getEt())) {
                    build.addField("st", clientReportErrorModel.getSt());
                    //插入原始数据表
                    buildData.addField("st", clientReportErrorModel.getSt());
                } else if ("PoC".equals(clientReportErrorModel.getEt()) || "NETWORK".equals(clientReportErrorModel.getEt())){
                    tag.put("ec", clientReportErrorModel.getEc());
                    tag.put("methodevent", clientReportErrorModel.getMethodevent());
                    build.addField("ei", clientReportErrorModel.getEi());
                    build.addField("body", StringUtils.isBlank(clientReportErrorModel.getBody())?" ":clientReportErrorModel.getBody());

                    //插入原始数据表
                    buildData.addField("ec", clientReportErrorModel.getEc());
                    buildData.addField("methodevent", clientReportErrorModel.getMethodevent());
                    buildData.addField("ei", clientReportErrorModel.getEi());
                    buildData.addField("body", StringUtils.isBlank(clientReportErrorModel.getBody())?" ":clientReportErrorModel.getBody());

                    //存入各个et的原始数据
                   // clientReportRedis.insertEiReport(clientReportErrorModel.getMethodevent(), clientReportErrorModel);
                }else {
                    buildData.addField("ei", clientReportErrorModel.getEi());
                    buildData.addField("body", StringUtils.isBlank(clientReportErrorModel.getBody())?" ":clientReportErrorModel.getBody());
                }
                build.addField("uid", clientReportErrorModel.getUid());
                build.addField("response", clientReportErrorModel.getResponse() == null ? "null" : clientReportErrorModel.getResponse());
                build.time(System.currentTimeMillis() * 1000000 + random.nextInt(999999), TimeUnit.NANOSECONDS);
                //插入原始数据表
                buildData.addField("uid", clientReportErrorModel.getUid());
                buildData.addField("param",clientReportErrorModel.getParam() == null? "null":clientReportErrorModel.getParam());
                buildData.addField("response", clientReportErrorModel.getResponse() == null ? "null" : clientReportErrorModel.getResponse());
                buildData.time(System.currentTimeMillis() * 1000000 + random.nextInt(999999), TimeUnit.NANOSECONDS);
                errorReportMapper.writeData(buildData.build());

                for (Map.Entry<String, String> entry : tag.entrySet()) {
                    if (StringUtils.isBlank(entry.getValue())) {
                        tag.put(entry.getKey(),"null");
                    }
                }
                build.tag(tag);
                //inlfuxdb 存入原始数据
                errorReportMapper.write(build.build());
                //存入错误类型原始数据
                clientReportRedis.insertExceptionReport(clientReportErrorModel);

                // 将数据存入hadoop fs里
                ReportCons.writer(clientReportErrorModel);
            }
            //存入用户原始数据
            if (StringUtils.isNotBlank(list.get(0).getUid())) {
                clientReportRedis.insertUidReport(list.get(0).getUid(), _mlogs);
            }

            //存入影响用户数 按照exceptiontype app版本 client 和 et
            if (StringUtils.isNotBlank(list.get(0).getExceptionType())) {
                clientReportRedis.insertEXEffectUid(list.get(0).getEt(), client, list.get(0).getUid(), list.get(0).getExceptionType(), list.get(0).getAv());
            }

        } catch (Exception e) {
            log.error("writeErrorData~~", e.toString());
        }

    }


}
