package com.yoho.opsmanager.other.ctrl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yoho.opsmanager.other.model.response.BaseRespone;
import com.yoho.opsmanager.other.notifier.DingDing;
import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;

/**
 *
 *   Grafana alert webhook. refer to:http://docs.grafana.org/alerting/notifications/
 *
 *   <pre>
 *
 *       {
            "title": "My alert",
            "ruleId": 1,
            "ruleName": "Load peaking!",
            "ruleUrl": "http://url.to.grafana/db/dashboard/my_dashboard?panelId=2",
            "state": "alerting",
            "imageUrl": "http://s3.image.url",
            "message": "Load is peaking. Make sure the traffic is real and spin up more webfronts",
            "evalMatches": [
                {
                    "metric": "requests",
                    "tags": {},
                    "value": 122
                 }
                ]
            }
 *   </pre>
 *
 * Created by chunhua.zhang on 2017/5/26.
 */
@Controller
public class GrafanaAlertController {
    Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private DingDing ding;

    @RequestMapping(value = "/ghook")
    @ResponseBody
    public BaseRespone  alert(HttpServletRequest request) {
        try {
            String jsonBody = IOUtils.toString(request.getInputStream());

            logger.info("receive grafana webhook message:{}", jsonBody);

            JSONObject alert = (JSONObject) JSON.parse(jsonBody);

            String title = alert.getString("title");
            String state = alert.getString("state");
            String imgUrl = alert.getString("imageUrl");
            String message = alert.getString("message");

            this.ding.publish(title, state, message, imgUrl);

        } catch (IOException e) {
            e.printStackTrace();
        }

        return BaseRespone.successResp(null);
    }




}
