package com.yoho.opsmanager.other.dns;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yoho.opsmanager.common.config.SystemConfig;
import com.yoho.opsmanager.common.util.ApplicationUtil;
import com.yoho.opsmanager.dao.mapper.ClientDnsMapper;
import com.yoho.opsmanager.global.service.AreaLocal;
import com.yoho.opsmanager.global.service.HttpRestClientService;
import com.yoho.opsmanager.other.DnsIPs;
import com.yoho.opsmanager.other.model.DnsRequest;
import net.sf.json.JSONArray;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.influxdb.dto.Point;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.UnsupportedEncodingException;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * Created by zhengyouwei on 2016/8/24.
 */
public class DnsWriteRunnable implements Runnable {

    private Logger log = LoggerFactory.getLogger(getClass());

    private static Random random = new Random();

    private ObjectMapper mapper = new ObjectMapper();


    private DnsRequest dnsRequest;

    public DnsWriteRunnable(DnsRequest dnsRequest) {
        this.dnsRequest = dnsRequest;
    }

    @Override
    public void run() {
        try {
            writeDnsData(dnsRequest.getArray(), dnsRequest.getClient());
        } catch (Exception e) {
            log.error("DnsWriteRunnable error", e);
        }

    }

    public void writeDnsData(JSONArray array, String client) {
        ClientDnsMapper clientDnsMapper = (ClientDnsMapper) ApplicationUtil.getBean(ClientDnsMapper.class);
        List<Point> pointList = dnsPoint(array, client);
        for (Point point : pointList) {
            clientDnsMapper.write(point);
        }
    }

    /**
     * dns
     *
     * @param array
     * @return 2016年5月18日下午3:27:19
     */
    public List<Point> dnsPoint(JSONArray array, String client) {
        List<Point> pointList = new ArrayList<Point>();
        try {
            for (Object error : array) {
                Map<String, Object> map = readValue(error.toString());
                String uid = (String) map.get("uid");
                List<Map<String, Object>> paramList = (List<Map<String, Object>>) map.get("param");
                if (CollectionUtils.isEmpty(paramList)) {
                    return pointList;
                }
                for (int i = 0; i < paramList.size(); i++) {
                    Point.Builder build = Point.measurement("client_dns_data");
                    boolean insert = true;
                    Map<String, Object> paramMap = paramList.get(i);
                    insert = paramBuild(paramMap, build);//配置文件不存在该域名则不分析
                    if (!insert) {
                        continue;
                    }
                    for (String key : map.keySet()) {
                        if (!insert) {
                            break;
                        }
                        switch (key) {
                            case "param":
                                break;
                            case "uid":
                                insert = getLocation(uid, build, paramMap);//获取不到省份城市，则不分析
                                break;
                            case "net":
                                String net = String.valueOf(map.get("net"));
                                if (net == "null" || StringUtils.isBlank(net)) {
                                    insert = false;
                                } else {
                                    build.tag(key, net);
                                }
                                break;
                            case "provider":
                                insert = getProvider(String.valueOf(map.get("provider")), paramMap, build);
                                break;
                            default:
                                String defaultvalue = String.valueOf(map.get(key));
                                build.addField(key, defaultvalue);
                        }
                    }
                    if (insert) {
                        build.tag("client", client);
                        build.time(System.currentTimeMillis() * 1000000 + random.nextInt(999999), TimeUnit.NANOSECONDS);
                        pointList.add(build.build());
                    }
                }
            }
        } catch (Exception ex) {
        }
        return pointList;
    }

    /**
     * uid locate
     *
     * @param build
     */
    private boolean getLocation(String uid, Point.Builder build, Map<String, Object> paramMap) {

        HttpRestClientService httpRestClientService = (HttpRestClientService) ApplicationUtil.getBean(HttpRestClientService.class);
        AreaLocal areaLocal = (AreaLocal) ApplicationUtil.getBean(AreaLocal.class);
        SystemConfig systemConfig = (SystemConfig) ApplicationUtil.getBean(SystemConfig.class);


        if (StringUtils.isBlank(uid)) {//空则尝试从百度获取
            Map<String, String> header = new HashMap<>();
            header.put("apikey", "bbc9420115e357d2d396b8707525653c");
            String result = httpRestClientService.doGet("http://apis.baidu.com/showapi_open_bus/ip/ip?ip=" + String.valueOf(paramMap.get("localdns")).split(",")[0], null, header);
            Map<String, Object> json = readValue(result);
            Map<String, Object> map = (Map<String, Object>) json.get("showapi_res_body");
            String region = (String) map.get("region");
            String city = (String) map.get("city");
            String regionCode = areaLocal.getCodeByname(region);
            if (regionCode == null) {
                return false;
            }
            String cityCode = areaLocal.getCodeByname(city);
            if (cityCode == null) {
                return false;
            }
            build.tag("province_code", regionCode);
            build.tag("city_code", cityCode);
            build.addField("province", region);
            build.addField("city", city);
            build.addField("uid", "null");
            return true;

        } else {//不为空则调用大数据
            //String locationResp = httpRestClientService.doGet(systemConfig.getBigdataurl() + uid, null);
            String locationResp = null;
            if (locationResp == null) {
                return false;
            }
            Map<String, Object> map = readValue(locationResp);
            if (!map.containsKey("provinceCode") || !map.containsKey("cityCode")) {
                return false;
            }
            String provinceCode = String.valueOf(map.get("provinceCode"));
            String cityCode = String.valueOf(map.get("cityCode"));
            String city = String.valueOf(map.get("city"));
            String province = String.valueOf(map.get("province"));
            build.tag("province_code", provinceCode);
            build.tag("city_code", cityCode);
            build.addField("province", province);
            build.addField("city", city);
            build.addField("uid", uid);
            return true;
        }
    }

    /**
     * 运营商校验
     *
     * @param build
     */
    private boolean getProvider(String provider, Map<String, Object> paramMap, Point.Builder build) throws UnsupportedEncodingException {
        if ("UNKNOW".equals(provider)) {
            HttpRestClientService httpRestClientService = (HttpRestClientService) ApplicationUtil.getBean(HttpRestClientService.class);
            Map<String, String> header = new HashMap<>();
            header.put("apikey", "bbc9420115e357d2d396b8707525653c");
            String result = httpRestClientService.doGet("http://apis.baidu.com/showapi_open_bus/ip/ip?ip=" + String.valueOf(paramMap.get("localdns")).split(",")[0], null, header);
            Map<String, Object> json = readValue(result);
            Map<String, Object> map = (Map<String, Object>) json.get("showapi_res_body");
            if ("电信".equals(String.valueOf(map.get("isp")))) {
                build.tag("provider", "CTCC");
            } else if ("联通".equals(String.valueOf(map.get("isp")))) {
                build.tag("provider", "CUCC");
            } else if ("移动".equals(String.valueOf(map.get("isp")))) {
                build.tag("provider", "CMCC");
            } else {
                build.tag("provider", "OTHER");
            }
        } else {
            build.tag("provider", provider);
        }
        return true;
    }

    /**
     * param
     *
     * @param build
     */
    private boolean paramBuild(Map<String, Object> map, Point.Builder build) {
        DnsIPs dnsIPs = (DnsIPs) ApplicationUtil.getBean(DnsIPs.class);

        for (String key : map.keySet()) {
            switch (key) {
                case "domain":
                    String value = String.valueOf(map.get(key));
                    build.tag(key, value);
                    break;
                default:
                    String value2 = String.valueOf(map.get(key));
                    build.addField(key, value2);
            }
        }

        //校验是否劫持
        String domain = String.valueOf(map.get("domain"));
        if (!dnsIPs.hasDomain(domain)) {
            return false;
        }
        String domainip = String.valueOf(map.get("domainip"));
        boolean result = dnsIPs.isLocalIp(domain, domainip);
        build.tag("is_real", String.valueOf(result));
        return true;
    }

    /**
     * json转化为Map
     *
     * @param json
     * @return 2016年5月19日下午4:21:06
     */
    public Map<String, Object> readValue(String json) {
        try {
            return mapper.readValue(json.toString(), new TypeReference<Map<String, Object>>() {
            });
        } catch (Exception e) {
            log.error("json parse to Map error.json:{}" + json);
            return null;
        }
    }


}
