package com.yoho.opsmanager.other.influxdb;

import com.yoho.opsmanager.common.model.ClientReport.ClientReportErrorModel;
import org.influxdb.InfluxDB;
import org.influxdb.InfluxDBFactory;
import org.influxdb.dto.BatchPoints;
import org.influxdb.dto.Point;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 *  写入数据到influxdb中
 * Created by chunhua.zhang on 2017/5/25.
 */
@Service
public class InfluxDBWriter {

    /**
     *  influxdb : 10.66.4.25:8086
     *  Database Name: app_errors
     *  Table Name： app_error_logs
     *  Retention Policy: default
     */
    private final String APP_ERROR_INFLUXDB_URL = "http://10.66.4.25:8086";
    private final String APP_ERROR_INFLUXDB_USER = "root";
    private final String APP_ERROR_INFLUXDB_PASSWORD= "root";
    private final String APP_ERROR_INFLUXDB_DB = "app_errors";
    private final String APP_ERROR_INFLUXDB_RP = "default";
    private final String APP_ERROR_INFLUXDB_TABLE = "app_error_logs";

    private final InfluxDB influxDB;
    public  InfluxDBWriter(){
          influxDB = InfluxDBFactory.connect(APP_ERROR_INFLUXDB_URL, APP_ERROR_INFLUXDB_USER, APP_ERROR_INFLUXDB_PASSWORD);
    }

    public void doWrite(List<ClientReportErrorModel> errors){
        if(errors == null){
            return;
        }
        BatchPoints batchPoints = BatchPoints
                .database(APP_ERROR_INFLUXDB_DB)
                .retentionPolicy(APP_ERROR_INFLUXDB_RP)
                .consistency(InfluxDB.ConsistencyLevel.ALL)
                .build();
        for(ClientReportErrorModel error: errors) {
            Point point = Point.measurement(APP_ERROR_INFLUXDB_TABLE)
                    .time(System.currentTimeMillis(), TimeUnit.MILLISECONDS)
                    .tag("dclass", error.getDevClass())  //android or ios
                    .tag("et", error.getEt())  //error type
                    .tag("osv", error.getOsv()) //os version
                    .tag("dm", error.getDm())  //device model
                    .tag("av", error.getAv()) //app version
                    .tag("ca", error.getCa())  //
                    .addField("udid", error.getUdid())
                    .build();
            batchPoints.point(point);
        }
        influxDB.write(batchPoints);
    }

}
