package com.yoho.opsmanager.other.notifier;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

/**
 *  Notifier to Dinging. refer to: https://open-doc.dingtalk.com/docs/doc.htm?spm=a219a.7629140.0.0.No1dRd&treeId=257&articleId=105735&docType=1
 * Created by chunhua.zhang on 2017/5/26.
 */
@Service
public class DingDing {
    Logger logger = LoggerFactory.getLogger(getClass());

    private final String WEBHOOK_TOKEN = "https://oapi.dingtalk.com/robot/send?access_token=31cf1a526c98862b7945e561c5a0d857a069350dbee0e807074b83b39fa1e1af";

    private  final  HttpClient httpclient;

    public DingDing(){
        httpclient = HttpClients.createDefault();
    }

    public void   publish(String title, String state, String message, String picURL){
        String msg;
        //普通消息
        if(StringUtils.isEmpty(picURL)){
             msg = this.getSimpleText(title, state, message);
        }else{
            msg = this.getFeedCardMessage(title, state, picURL);
        }

        try {
            this.publish(msg);
        } catch (IOException e) {
            logger.error("push error at:" + msg, e);
        }
    }



    private void publish(String textMsg) throws IOException {
        HttpPost httppost = new HttpPost(WEBHOOK_TOKEN);
        httppost.addHeader("Content-Type", "application/json; charset=utf-8");

        StringEntity se = new StringEntity(textMsg, "utf-8");
        httppost.setEntity(se);
        logger.info("send Ding feedcard message :{}", textMsg);
        HttpResponse response = httpclient.execute(httppost);
        if (response.getStatusLine().getStatusCode()== HttpStatus.SC_OK){
            String result= EntityUtils.toString(response.getEntity(), "utf-8");
            logger.info("get ding return: " + result);
        }
    }

    /**
     * {
     "msgtype": "text",
     "text": {
     "content": "我就是我, 是不一样的烟火"
     },
     "at": {
     "atMobiles": [
     "156xxxx8827",
     "189xxxx8325"
     ],
     "isAtAll": false
     }
     }
     */
    private  String  getSimpleText(String title, String state, String message ){

        JSONObject root = new JSONObject();
        root.put("msgtype", "text");

        JSONObject text = new JSONObject();
        text.put("content", state + ":" + title + ":" + message);

        root.put("text", text);

        return root.toJSONString();
    }

    /**
     *
     *  Dingding Freecard 消息
      "feedCard": {
     "links": [
     {
     "title": "时代的火车向前开",
     "messageURL": "www.baidu.com",
     "picURL": "https://s3.cn-north-1.amazonaws.com.cn/grafana-alert/70puoQB6OiDEJZPH6E9F.png"
     }
     ]
     },
     "msgtype": "feedCard"
     }
     */

    private  String getFeedCardMessage(String title,  String state, String picURL){

        JSONObject root = new JSONObject();
        root.put("msgtype", "feedCard");

        JSONObject feedCard = new JSONObject();
        root.put("feedCard", feedCard);

          //one card
        JSONObject card = new JSONObject();
        card.put("title",   title );


        /***
         *  bug fix for:  https://github.com/grafana/grafana/issues/8397
         *  we must change the url from https://grafana-alert.s3-cn-north-1.amazonaws.com/D0gtSOTbjt65tJ0bGmwj.png
         *  to  https://s3.cn-north-1.amazonaws.com.cn/grafana-alert/70puoQB6OiDEJZPH6E9F.png
         */
        try {
            URL url = new URL(picURL);
            String path = url.getPath();
            String host = url.getHost();

            String[] hosts = host.split("\\.");
            String newURL =  "https://s3.cn-north-1.amazonaws.com.cn/"+hosts[0] + path;
            card.put("picURL", newURL);
            card.put("messageURL", newURL);
        } catch (MalformedURLException e) {
            e.printStackTrace();
        }

        JSONArray cards = new JSONArray();
        cards.add(card);

        feedCard.put("links", cards);
        return root.toJSONString();
    }



}


