package com.yoho.opsmanager.redis.service;

import com.alibaba.fastjson.JSON;
import com.sun.corba.se.spi.ior.IdentifiableFactory;
import com.yoho.opsmanager.common.model.ClientReport.ClientReportErrorModel;
import com.yoho.opsmanager.common.model.ClientReport.UidStatus;
import com.yoho.opsmanager.redis.common.RedisKeyPre;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.data.redis.connection.RedisConnection;
import org.springframework.data.redis.core.RedisCallback;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.TimeoutUtils;
import org.springframework.stereotype.Component;

import java.io.UnsupportedEncodingException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * Created by zhengyouwei on 2016/9/13.
 */
@Component
public class ClientReportRedis {

    @Autowired
    @Qualifier("clientRedisTemplate")
    private RedisTemplate redisTemplate;

    Logger logger = LoggerFactory.getLogger(getClass());

    /**
     * 存储UID 的error
     *
     * @param report
     */
    public void insertUidReport(String uid, String report) {
        try {
            redisTemplate.opsForList().leftPush(RedisKeyPre.CLIENT_UID_PRE + uid, report);
        } catch (Exception e) {
            logger.error("insertUidReport~", e);
        }
        try {
            //每个uid保存400个数据
            redisTemplate.opsForList().trim(RedisKeyPre.CLIENT_UID_PRE + uid, 0, 400);
        } catch (Exception e) {
            logger.error("insertUidReport~", e);
        }
        try {
            //每个uid 20天有效期
            this.longExpire(RedisKeyPre.CLIENT_UID_PRE + uid, 20, TimeUnit.DAYS);
        } catch (Exception e) {
            logger.error("insertUidReport~", e);
        }


    }

//    /**
//     * 存储UID 对应的ET 的error
//     *
//     * @param report
//     */
//    public void insertUidReport(String uid, String report, String et) {
//        redisTemplate.opsForList().leftPush(RedisKeyPre.CLIENT_UID_PRE + et + "." + uid, report);
//        //每个uid保存500个数据
//        redisTemplate.opsForList().trim(RedisKeyPre.CLIENT_UID_PRE + et + "." + uid, 0, 50);
//        //每个uid 30天有效期
//        this.longExpire(RedisKeyPre.CLIENT_UID_PRE + et + "." + uid, 20, TimeUnit.DAYS);
//    }

//    /**
//     * 上报事件存储
//     *
//     * @param report
//     */
//    public void insertEiReport(String ei, ClientReportErrorModel report) {
//        redisTemplate.opsForList().leftPush(RedisKeyPre.CLIENT_EI_PRE + ei, JSON.toJSONString(report));
//        //每个ei保存40条
//        redisTemplate.opsForList().trim(RedisKeyPre.CLIENT_EI_PRE + ei, 0, 20);
//        //每个ei 1天有效期
//        this.longExpire(RedisKeyPre.CLIENT_EI_PRE + ei, 2, TimeUnit.DAYS);
//    }

    /**
     * 上报事件存储
     *
     * @param report
     */
    public void insertExceptionReport(ClientReportErrorModel report) {

        if (StringUtils.isNotBlank(report.getExceptionType())) {
            String key = report.getEt() + "." + report.getClient() + "." + report.getExceptionType();
            try {
                redisTemplate.opsForList().leftPush(RedisKeyPre.CLIENT_EXCEPTION_PRE + key, JSON.toJSONString(report));
            } catch (Exception e) {
                logger.error("insertExceptionReport~", e);
            }
            try {
                //每个Exception保存20条
                redisTemplate.opsForList().trim(RedisKeyPre.CLIENT_EXCEPTION_PRE + key, 0, 20);
            } catch (Exception e) {
                logger.error("insertExceptionReport~", e);
            }
            try {
                //每个Exception 10天有效期
                this.longExpire(RedisKeyPre.CLIENT_EXCEPTION_PRE + key, 10, TimeUnit.DAYS);
            } catch (Exception e) {
                logger.error("insertExceptionReport~", e);
            }

        }
    }

//    /**
//     * 插入et影响用户数据
//     *
//     * @param et
//     * @param client
//     * @param uid
//     */
//    public void insertEffectUid(String et, String client, String uid) {
//        String key = RedisKeyPre.CLIENT_ET_EFFECT_PRE + et + "." + client;
//        if (!redisTemplate.opsForSet().isMember(key, uid)) {
//            redisTemplate.opsForSet().add(key, uid);
//        }
//    }

    /**
     * 插入exception影响用户数据
     *
     * @param et
     * @param client
     * @param uid
     */
    public void insertEXEffectUid(String et, String client, String uid, String exception, String av) {
//        if ("CRASH".equals(et)) {
//            String key = RedisKeyPre.CLIENT_EX_EFFECT_PRE + et + "." + client + "." + exception + "." + av;
//            if (!redisTemplate.opsForSet().isMember(key, uid)) {
//                redisTemplate.opsForSet().add(key, uid);
//                String key2 = RedisKeyPre.CLIENT_EX_EFFECT_PRE + et + "." + client + "." + exception;
//                redisTemplate.opsForHash().increment(key2, "all", 1);
//                redisTemplate.opsForHash().increment(key2, av, 1);
//                redisTemplate.opsForHash().put(key2, av + "lasttime", new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()));
//            }
//        } else {
//            String key2 = RedisKeyPre.CLIENT_EX_EFFECT_PRE + et + "." + client + "." + exception;
//            redisTemplate.opsForHash().increment(key2, "all", 1);
//            redisTemplate.opsForHash().increment(key2, av, 1);
//            redisTemplate.opsForHash().put(key2, av + "lasttime", new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()));
//        }


//
        // String key = RedisKeyPre.CLIENT_EX_EFFECT_PRE + et + "." + client + "." + exception + "." + av;
        //if (!redisTemplate.opsForSet().isMember(key, uid)) {
        // redisTemplate.opsForSet().add(key, uid);
        String key2 = RedisKeyPre.CLIENT_EX_EFFECT_PRE + et + "." + client + "." + exception;
        try {
            redisTemplate.opsForHash().increment(key2, "all", 1);

        } catch (Exception e) {
            logger.error("insertEXEffectUid~", e);
        }
        try {
            redisTemplate.opsForHash().increment(key2, av, 1);

        } catch (Exception e) {
            logger.error("insertEXEffectUid~", e);
        }
        try {
            redisTemplate.opsForHash().put(key2, av + "lasttime", new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()));

        } catch (Exception e) {
            logger.error("insertEXEffectUid~", e);
        }

        //  }
    }

    /**
     * 更新用户的信息
     *
     * @param report
     */
    public void updateUidInfo(ClientReportErrorModel report) {
        if (StringUtils.isBlank(report.getUid())) {
            return;
        }
        UidStatus uidStatus = new UidStatus();
        uidStatus.setAv(report.getAv());
        uidStatus.setOsv(report.getOsv());
        uidStatus.setDm(report.getDm());
        uidStatus.setClient(report.getClient());
        uidStatus.setCa(report.getCa());

        String key = RedisKeyPre.CLIENT_UID_STATUS_PRE + report.getUid();
        Object o = redisTemplate.opsForValue().get(key);
        if (o != null) {
            String str = o.toString();
            if (StringUtils.isNotBlank(str)) {
                UidStatus uidStatus1 = JSON.parseObject(str, UidStatus.class);
                if (uidStatus.equals(uidStatus1)) {
                    return;
                }
            }
        }
        //更新用户数据
        redisTemplate.opsForValue().set(key, JSON.toJSONString(uidStatus));

    }

    /**
     * spring redis 设置超长时间expire，会调用twemproxy不支持的time命令，导致报错。
     *
     * @param key
     * @param timeout
     * @param unit
     * @return
     */
    public Boolean longExpire(String key, final long timeout, final TimeUnit unit) {

        try {
            final byte[] rawKey = String.valueOf(key).getBytes("UTF-8");

            return (Boolean) redisTemplate.execute(new RedisCallback() {
                @Override
                public Boolean doInRedis(RedisConnection connection) {
                    return connection.expire(rawKey, TimeoutUtils.toSeconds(timeout, unit));
                }
            }, true);
        } catch (UnsupportedEncodingException e) {
        }

        return false;
    }

}
