package com.yoho.opsmanager.task;

import com.yoho.opsmanager.common.contants.DataBaseContants;
import com.yoho.opsmanager.common.util.QueryResultUtil;
import com.yoho.opsmanager.dao.mapper.ClientDnsMapper;
import org.influxdb.dto.BatchPoints;
import org.influxdb.dto.Point;
import org.influxdb.dto.QueryResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Random;
import java.util.concurrent.TimeUnit;

@Component
public class ClientDnsReportTask {

    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private ClientDnsMapper clientDnsMapper;

    private static Random random = new Random();

    @Scheduled(cron = "${cron.task.client.dns}")
    public void clientDnsCountMonitor() {
        try {
            BatchPoints batchPoints = BatchPoints.database(DataBaseContants.YOMO_DNS_REPORT).retentionPolicy("one_month").build();

            QueryResult queryResult = clientDnsMapper.getDataOneHour();
            List<QueryResult.Series> series = QueryResultUtil.getSeries(queryResult);
            if (series == null) {
                return;
            }
            try {
                for (QueryResult.Series iSeries : series) {
                    int count = iSeries.getValues().get(0).get(1) == null ?
                            0 : (int) (double) iSeries.getValues().get(0).get(1);
                    String domain = iSeries.getTags().get("domain");
                    String is_real = iSeries.getTags().get("is_real");
                    String net = iSeries.getTags().get("net");
                    String provider = iSeries.getTags().get("provider");
                    String province_code = iSeries.getTags().get("province_code");
                    String city_code = iSeries.getTags().get("city_code");


                    Point.Builder build = Point.measurement("client_dns_hour");
                    build.addField("count",count);
                    build.tag("domain",domain);
                    build.tag("is_real",is_real);
                    build.tag("net",net);
                    build.tag("provider",provider);
                    build.tag("province_code",province_code);
                    build.tag("city_code",city_code);
                    build.time(System.currentTimeMillis() * 1000000 + random.nextInt(999999), TimeUnit.NANOSECONDS);
                    batchPoints.point(build.build());
                }
                clientDnsMapper.write(batchPoints);
            } catch (Exception e) {
                logger.error("clientDnsCountMonitor failure!" + e.toString());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
