package com.yoho.opsmanager.task.service.aop;

import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@Aspect
public class TaskLogAspect {

	private static final Logger log = LoggerFactory.getLogger(TaskLogAspect.class);

	@Around("execution (@org.springframework.scheduling.annotation.Scheduled  * *.*(..))")
	public Object traceBackgroundThread(final ProceedingJoinPoint pjp) throws Throwable {
		Signature signature = pjp.getSignature();
		long start = System.currentTimeMillis();
		try {
			return pjp.proceed();
		}
		finally {
			long end = System.currentTimeMillis();
			log.debug("task end class:{},methed:{},cost:{}",signature.getDeclaringTypeName(),signature.getName(),(end-start));
		}

	}

}
