package com.yoho.opsmanager.task.service.impl;


import com.yoho.opsmanager.common.contants.DataBaseContants;
import com.yoho.opsmanager.common.contants.InfluxDBName;
import com.yoho.opsmanager.common.util.ClientReportUtil;
import com.yoho.opsmanager.common.util.QueryResultUtil;
import com.yoho.opsmanager.common.util.ReportCount;
import com.yoho.opsmanager.dao.mapper.ClientReportMapper;
import com.yoho.opsmanager.dao.mapper.MonitorAlarmMapper;
import com.yoho.opsmanager.global.service.AlarmMsgService;
import com.yoho.opsmanager.task.service.ClientReportErrorService;
import org.influxdb.dto.BatchPoints;
import org.influxdb.dto.Point;
import org.influxdb.dto.QueryResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;

@Service
public class ClientReportErrorServiceImpl implements ClientReportErrorService {

    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private AlarmMsgService alarmMsgService;

    @Autowired
    private MonitorAlarmMapper monitorAlarmMapper;

    @Autowired
    private ClientReportMapper clientReportMapper;

    private static int lastNum = 0;

    private static int alarmlast = 0;

    private static int insertlast = 0;

    @Value("${clien_error_mobile}")
    private String mobile;


    /**
     * 监控告警
     *
     * @param minute
     * @param flag
     */
    @Override
    public void clentReportErrorMonitor(int minute, int flag) {
        int count = ReportCount.getIncrement();
        int increate = count - alarmlast;
        alarmlast = count;
        if (lastNum != 0 && increate != 0 && increate > flag && increate > 2 * lastNum && ReportCount.getErrorSum() > 10000) {
            alarm(InfluxDBName.AWS, increate, minute);
            ReportCount.setErrorSum(0);
        }
        lastNum = increate;

        //昨天同比
        QueryResult queryResult = clientReportMapper.getYestodayCount();
        int yestodayCount = QueryResultUtil.getCount(queryResult);
        if (yestodayCount == 0) {
            return;
        }
        if ((increate > yestodayCount * 3) && (ReportCount.getErrorSum() > 10000)) {
            alarm(increate, minute, yestodayCount);
            ReportCount.setErrorSum(0);
        }
    }

    /**
     * 每隔20S统计一下数量
     */
    @Override
    public void clientErrorGroupInsert() {
        int count = ReportCount.getIncrement();
        int increate = count - insertlast;
        insertlast = count;
        BatchPoints batchPoints = BatchPoints.database(DataBaseContants.YOMO_REPORT).retentionPolicy("ten_day").build();
        org.influxdb.dto.Point.Builder build = Point.measurement("client_report_grafana");
        build.addField("count", increate);
        batchPoints.point(build.build());
        clientReportMapper.write(batchPoints);
    }


    /**
     * 每小时统计
     */
    @Override
    public void clientHourInsert() {
        List<QueryResult> list = new ArrayList<>();
        list.add(getQueryResult("IMAGELOAD"));
        list.add(getQueryResult("NETWORK"));
        list.add(getQueryResult("PoC"));
        list.add(getQueryResult("CRASH"));

        String date = new SimpleDateFormat("yyyyMMddHH").format(new Date());

        for (QueryResult queryResult : list) {
            List<QueryResult.Series> series = QueryResultUtil.getSeries(queryResult);
            if (series == null) {
                continue;
            }
            try {
                BatchPoints batchPoints = BatchPoints.database(DataBaseContants.YOMO_REPORT).retentionPolicy("ten_day").build();
                for (QueryResult.Series iSeries : series) {
                    int count = iSeries.getValues().get(0).get(1) == null ?
                            0 : (int) (double) iSeries.getValues().get(0).get(1);
                    String dm = iSeries.getTags().get("dm");
                    String av = iSeries.getTags().get("av");
                    String client = iSeries.getTags().get("client");
                    String osv = iSeries.getTags().get("osv");
                    String et = iSeries.getTags().get("et");
                    String exceptionType = iSeries.getTags().get("exceptionType");


                    org.influxdb.dto.Point.Builder build = Point.measurement(ClientReportUtil.getHourName(et,client));
                    QueryResult queryResult1 = clientReportMapper.selectEffectUid(ClientReportUtil.getTableName(et), dm, av, client, osv, exceptionType);
                    int udidCount = QueryResultUtil.getCount(queryResult1);
                    build.tag("dm", dm);
                    build.tag("av", av);
                    build.tag("osv", osv);
                    build.tag("date", date);
                    build.tag("exceptionType", exceptionType);
                    build.addField("exceptionCount", count);
                    build.addField("udidCount", udidCount);
                    batchPoints.point(build.build());
                }
                clientReportMapper.write(batchPoints);
            } catch (Exception e) {
                logger.error("clientHourInsert failure!" + e.toString());
            }
        }
    }

    public QueryResult getQueryResult(String et) {
        String table = ClientReportUtil.getTableName(et);
        QueryResult queryResult = clientReportMapper.selectClientByGroup(table, 60);
        return queryResult;
    }

    /**
     * 每天统计
     */
    @Override
    public void clientDayInsert() {
        addDayDate("IMAGELOAD","ios");
        addDayDate("NETWORK","ios");
        addDayDate("PoC","ios");
        addDayDate("CRASH","ios");

        addDayDate("IMAGELOAD","android");
        addDayDate("NETWORK","android");
        addDayDate("PoC","android");
        addDayDate("CRASH","android");

    }

    public void addDayDate(String et,String client){

        try {
            String date = new SimpleDateFormat("yyyyMMdd").format(new Date());

            QueryResult queryResult = clientReportMapper.selectByDay(ClientReportUtil.getHourName(et,client));
            List<QueryResult.Series> series = QueryResultUtil.getSeries(queryResult);
            if (series == null) {
                return;
            }

            BatchPoints batchPoints = BatchPoints.database(DataBaseContants.YOMO_REPORT).retentionPolicy("one_year").build();
            for (QueryResult.Series iSeries : series) {
                int exceptionCount = iSeries.getValues().get(0).get(1) == null ?
                        0 : (int) (double) iSeries.getValues().get(0).get(1);
                int udidCount = iSeries.getValues().get(0).get(1) == null ?
                        0 : (int) (double) iSeries.getValues().get(0).get(2);
                String dm = iSeries.getTags().get("dm");
                String av = iSeries.getTags().get("av");
                String osv = iSeries.getTags().get("osv");
                String exceptionType = iSeries.getTags().get("exceptionType");

                org.influxdb.dto.Point.Builder build = Point.measurement(ClientReportUtil.getDayName(et, client));
                build.tag("dm", dm);
                build.tag("av", av);
                build.tag("osv", osv);
                build.tag("date", date);
                build.tag("exceptionType", exceptionType);
                build.addField("exceptionCount", exceptionCount);
                build.addField("udidCount", udidCount);
                batchPoints.point(build.build());
            }
            clientReportMapper.write(batchPoints);
        } catch (Exception e) {
            logger.error("clientDayInsert failure!" + e.toString());
        }
    }


    /**
     * 告警并且插入相应的告警数据到influxdb中
     *
     * @param influxDBName
     * @param minute
     */
    private void alarm(String influxDBName, int count, int minute) {
        String smsContent = buildNoCallbackAlarmSmsContent(count, minute);
        alarmMsgService.sendSms("client report", smsContent, mobile);
    }

    private String buildNoCallbackAlarmSmsContent(int count, int minute) {
        StringBuilder sb = new StringBuilder();
        sb.append("最近").append(minute).append("分钟内").append("client 上报了错误").append(count).append("次,");
        sb.append("上次上报").append(lastNum).append("次");
        return sb.toString();
    }

    /**
     * 告警并且插入相应的告警数据到influxdb中
     *
     * @param minute
     */
    private void alarm(int count, int minute, int yesterday) {
        String smsContent = buildNoCallbackAlarmSmsContent(count, minute, yesterday);
        alarmMsgService.sendSms("client report", smsContent, mobile);
    }

    private String buildNoCallbackAlarmSmsContent(int count, int minute, int yesterday) {
        StringBuilder sb = new StringBuilder();
        sb.append("最近").append(minute).append("分钟内").append("client 上报了错误").append(count).append("次,");
        sb.append("昨天同比上报").append(yesterday).append("次");
        return sb.toString();
    }


}
